from dataclasses import dataclass
import tempfile
from typing import Dict, Any
import yaml
from py_linq import Enumerable
from subprocess import Popen, PIPE


class ProcessRunException(Exception):
    pass


@dataclass
class Chart:
    chart: str
    binary: str = "helm"
    kube_version: str = ""
    api_version: str = ""
    namespace: str = "default"

    def _render_by_helm(self, *values: Dict[str, Any]):
        cmds = [self.binary, "template", self.chart]

        if self.kube_version:
            cmds.append(f"--kube-version={self.kube_version}")

        if self.api_version:
            cmds.append(f"--api-version={self.api_version}")

        if self.namespace:
            cmds.append(f"--namespace={self.namespace}")

        with tempfile.TemporaryDirectory() as tmpdir:
            for index, value in enumerate(values):
                value_file = f"{tmpdir}/values-{index}.yaml"
                cmds.append(f"--values={value_file}")
                with open(value_file, "w") as f:
                    yaml.safe_dump(value, f)

            process = Popen(cmds, stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()

        if process.poll() != 0:
            raise ProcessRunException(stderr)

        return stdout

    def render(self, *values: Dict[str, Any]):
        rendered = self._render_by_helm(*values)
        return Enumerable(yaml.safe_load_all(rendered))



import jmespath
from py_linq import Enumerable
import re


class KindFilter:
    def __init__(self, kind: str):
        self._kind = kind
        self._path = jmespath.compile("kind")

    def name(self, name: str):
        name_path = jmespath.compile("metadata.name")

        def match(x: Enumerable) -> bool:
            if not self(x):
                return False

            if name_path.search(x) != name:
                return False

            return True

        return match

    def name_match(self, pattern: str):
        name_path = jmespath.compile("metadata.name")
        name_re = re.compile(pattern)

        def match(x: Enumerable) -> bool:
            if not self(x):
                return False

            result = name_path.search(x)
            if result is None:
                return False

            return name_re.search(result) is not None

        return match

    def match_labels(self, expected_labels: Dict[str, str]):
        labels_path = jmespath.compile("metadata.labels")

        def match(x: Enumerable) -> bool:
            if not self(x):
                return False

            labels = labels_path.search(x)
            if labels is None:
                return False

            return not expected_labels.items() - labels.items()

        return match

    def __call__(self, x: Enumerable) -> bool:
            return self._path.search(x) == self._kind


class KubeKindFilter:
    service = KindFilter("Service")
    deployment = KindFilter("Deployment")
    stateful_set = KindFilter("StatefulSet")
    daemon_set = KindFilter("DaemonSet")
    job = KindFilter("Job")
    cron_job = KindFilter("CronJob")
    pod = KindFilter("Pod")
    persistent_volume_claim = KindFilter("PersistentVolumeClaim")
    persistent_volume = KindFilter("PersistentVolume")
    secret = KindFilter("Secret")
    config_map = KindFilter("ConfigMap")
    ingress = KindFilter("Ingress")
    service_account = KindFilter("ServiceAccount")
    role = KindFilter("Role")
    role_binding = KindFilter("RoleBinding")
    cluster_role = KindFilter("ClusterRole")
    cluster_role_binding = KindFilter("ClusterRoleBinding")
    custom_resource_definition = KindFilter("CustomResourceDefinition")
    service_monitor = KindFilter("ServiceMonitor")



import jmespath
from typing import Any
import re


class JMESPath:
    _compiled = None

    def __init__(self, expr: str):
        self._expr = expr

    def __getitem__(self, item: Any) -> Any:
        cls = self.__class__

        if isinstance(item, int):
            return cls(f"{self._expr}[{item}]")

        return cls(f"{self._expr}.{item}")

    def __call__(self, x: Any) -> Any:
        if self._compiled is None:
            self._compiled = jmespath.compile(self._expr)

        return self._compiled.search(x)

    def __eq__(self, other: Any):
        if isinstance(other, JMESPath):
            return self._expr == other._expr

        def match(x: Any) -> bool:
            return self(x) == other

        return match

    def __ne__(self, other: Any) -> bool:
        if isinstance(other, JMESPath):
            return self._expr == other._expr

        def match(x: Any) -> bool:
            return self(x) != other

        return match

    def __lt__(self, other: Any):
        def match(x: Any) -> bool:
            return self(x) < other

        return match

    def __le__(self, other: Any):
        def match(x: Any) -> bool:
            return self(x) <= other

        return match

    def __gt__(self, other: Any):
        def match(x: Any) -> bool:
            return self(x) > other

        return match

    def __ge__(self, other: Any):
        def match(x: Any) -> bool:
            return self(x) >= other

        return match

    def regexp(self, pattern: str):
        re_pattern = re.compile(pattern)

        def match(x: Any) -> bool:
            return re_pattern.search(self(x)) is not None

        return match

    def is_none(self):
        def match(x: Any) -> bool:
            return self(x) is None

        return match

    def is_not_none(self):
        def match(x: Any) -> bool:
            return self(x) is not None

        return match
